@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -25,13 +25,14 @@
 #include <gmodule.h>
 #include <libgda/gda-client.h>
 #include <libgda/gda-config.h>
-#include <libgda/gda-intl.h>
+#include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-server-provider.h>
 #include <string.h>
 #include "gda-marshal.h"
-
-#define PARENT_TYPE G_TYPE_OBJECT
+#include "gda-dict.h"
+#include "gda-parameter-list.h"
+#include "gda-value.h"
 
 typedef struct {
 	GModule              *handle;
@@ -40,7 +41,6 @@ typedef struct {
 	/* entry points to the plugin */
 	const gchar        *(*plugin_get_name) (void);
 	const gchar        *(*plugin_get_description) (void);
-	GList              *(*plugin_get_connection_params) (void);
 	GdaServerProvider  *(*plugin_create_provider) (void);
 	gchar              *(*get_dsn_spec) (void);
 } LoadedProvider;
@@ -54,7 +54,8 @@ static void gda_client_class_init (GdaClientClass *klass);
 static void gda_client_init       (GdaClient *client, GdaClientClass *klass);
 static void gda_client_finalize   (GObject *object);
 
-static void connection_error_cb (GdaConnection *cnc, GdaConnectionEvent *error, GdaClient *client);
+static void cnc_error_cb (GdaConnection *cnc, GdaConnectionEvent *error, GdaClient *client);
+static void cnc_destroyed_cb (GdaConnection *cnc, GdaClient *client);
 
 enum {
 	EVENT_NOTIFICATION,
@@ -108,7 +109,7 @@ free_hash_provider (gpointer key, gpointer value, gpointer user_data)
  */
 
 static void
-connection_error_cb (GdaConnection *cnc, GdaConnectionEvent *error, GdaClient *client)
+cnc_error_cb (GdaConnection *cnc, GdaConnectionEvent *error, GdaClient *client)
 {
 	g_return_if_fail (GDA_IS_CLIENT (client));
 
@@ -117,15 +118,15 @@ connection_error_cb (GdaConnection *cnc, GdaConnectionEvent *error, GdaClient *c
 }
 
 static void
-cnc_weak_cb (gpointer user_data, GObject *object)
+cnc_destroyed_cb (GdaConnection *cnc, GdaClient *client)
 {
-	GdaConnection *cnc = (GdaConnection *) object;
-	GdaClient *client = (GdaClient *) user_data;
-
-	g_return_if_fail (GDA_IS_CONNECTION (cnc));
-	g_return_if_fail (GDA_IS_CLIENT (client));
-
+	g_assert (g_list_find (client->priv->connections, cnc));
+	g_signal_handlers_disconnect_by_func (G_OBJECT (cnc), 
+					      G_CALLBACK (cnc_destroyed_cb), client);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (cnc), 
+					      G_CALLBACK (cnc_error_cb), client);
 	client->priv->connections = g_list_remove (client->priv->connections, cnc);
+	g_object_unref (cnc);
 }
 
 typedef struct {
@@ -214,7 +215,8 @@ gda_client_finalize (GObject *object)
 	g_return_if_fail (GDA_IS_CLIENT (client));
 
 	/* free memory */
-	gda_client_close_all_connections (client);
+	while (client->priv->connections)
+		cnc_destroyed_cb (GDA_CONNECTION (client->priv->connections->data), client);
 
 	g_hash_table_foreach (client->priv->providers, (GHFunc) remove_weak_ref, client);
 	g_hash_table_foreach (client->priv->providers, (GHFunc) free_hash_provider, NULL);
@@ -308,8 +310,6 @@ find_or_load_provider (GdaClient *client, const gchar *provider)
 			 (gpointer) &prv->plugin_get_name);
 	g_module_symbol (prv->handle, "plugin_get_description",
 			 (gpointer) &prv->plugin_get_description);
-	g_module_symbol (prv->handle, "plugin_get_connection_params",
-			 (gpointer) &prv->plugin_get_connection_params);
 	g_module_symbol (prv->handle, "plugin_create_provider",
 			 (gpointer) &prv->plugin_create_provider);
 	g_module_symbol (prv->handle, "plugin_get_dsn_spec",
@@ -340,22 +340,60 @@ find_or_load_provider (GdaClient *client, const gchar *provider)
 	return prv;
 }
 
+/**
+ * gda_client_declare_connection
+ * @client: a #GdaClient object
+ * @cnc: a #GdaConnection object
+ *
+ * Declares the @cnc to @client. This function should not be used directly
+ */
+void
+gda_client_declare_connection (GdaClient *client, GdaConnection *cnc)
+{
+	g_return_if_fail (client && GDA_IS_CLIENT (client));
+	g_return_if_fail (client->priv);
+	g_return_if_fail (cnc && GDA_IS_CONNECTION (cnc));
+	g_return_if_fail (cnc->priv);
+	
+	if (g_list_find (client->priv->connections, cnc))
+		return;
+
+	client->priv->connections = g_list_append (client->priv->connections, cnc);
+	g_object_ref (cnc);
+
+	/* signals */
+	g_signal_connect (G_OBJECT (cnc), "destroyed",
+			  G_CALLBACK (cnc_destroyed_cb), client);
+
+	g_signal_connect (G_OBJECT (cnc), "error",
+			  G_CALLBACK (cnc_error_cb), client);
+}
+
 /**
  * gda_client_open_connection
  * @client: a #GdaClient object.
  * @dsn: data source name.
- * @username: user name.
- * @password: password for @username.
+ * @username: user name or %NULL
+ * @password: password for @username, or %NULL
  * @options: options for the connection (see #GdaConnectionOptions).
  * @error: a place to store an error, or %NULL
  *
+ * This function is the way of opening database connections with libgda.
+ *
  * Establishes a connection to a data source. The connection will be opened
  * if no identical connection is available in the #GdaClient connection pool,
  * and re-used if available. If you dont want to share the connection,
  * specify #GDA_CONNECTION_OPTIONS_DONT_SHARE as one of the flags in
  * the @options parameter.
  *
- * This function is the way of opening database connections with libgda.
+ * The username and password used to actually open the connection are the first
+ * non-NULL string being chosen by order from
+ * <itemizedlist>
+ *  <listitem>the @username or @password</listitem>
+ *  <listitem>the username or password sprcified in the DSN definition</listitem>
+ *  <listitem>the USERNAME= and PASSWORD= parts of the connection string in the DSN definition</listitem>
+ * </itemizedlist>
+ *
  *
  * Returns: the opened connection if successful, %NULL if there is
  * an error.
@@ -403,14 +441,10 @@ gda_client_open_connection (GdaClient *client,
 	
 		if (prv) {
 			cnc = gda_connection_new (client, prv->provider, dsn, username, password, 
-						  options, error);
-
-			if (cnc) {
-				/* add cnc to our private list */
-				client->priv->connections = g_list_append (client->priv->connections, cnc);
-				g_object_weak_ref (G_OBJECT (cnc), (GWeakNotify) cnc_weak_cb, client);
-				g_signal_connect (G_OBJECT (cnc), "error",
-						  G_CALLBACK (connection_error_cb), client);
+						  options);
+			if (!gda_connection_open (cnc, error)) {
+				g_object_unref (cnc);
+				cnc = NULL;
 			}
 		}
 		else 
@@ -424,7 +458,6 @@ gda_client_open_connection (GdaClient *client,
 			     _("Datasource configuration error: no provider specified"));
 	}
 
-
 	/* free memory */
 	gda_data_source_info_free (dsn_info);
 
@@ -436,17 +469,28 @@ gda_client_open_connection (GdaClient *client,
  * @client: a #GdaClient object.
  * @provider_id: provider ID to connect to.
  * @cnc_string: connection string.
+ * @username: user name.
+ * @password: password for @username.
  * @options: options for the connection (see #GdaConnectionOptions).
  * @error: a place to store an error, or %NULL
  *
  * Opens a connection given a provider ID and a connection string. This
  * allows applications to open connections without having to create
  * a data source in the configuration. The format of @cnc_string is
- * similar to PostgreSQL and MySQL connection strings. It is a ;-separated
- * series of key=value pairs. Do not add extra whitespace after the ; 
+ * similar to PostgreSQL and MySQL connection strings. It is a semicolumn-separated
+ * series of key=value pairs. Do not add extra whitespace after the semicolumn
  * separator. The possible keys depend on the provider, but
  * these keys should work with all providers:
  * USER, PASSWORD, HOST, DATABASE, PORT
+ *
+ * The username and password used to actually open the connection are the first
+ * non-NULL string being chosen by order from
+ * <itemizedlist>
+ *  <listitem>the @username or @password</listitem>
+ *  <listitem>the USERNAME= and PASSWORD= parts of the @cnc_string</listitem>
+ * </itemizedlist>
+ *
+
  *
  * Returns: the opened connection if successful, %NULL if there is
  * an error.
@@ -455,12 +499,13 @@ GdaConnection *
 gda_client_open_connection_from_string (GdaClient *client,
 					const gchar *provider_id,
 					const gchar *cnc_string,
+					const gchar *username,
+					const gchar *password,
 					GdaConnectionOptions options,
 					GError **error)
 {
-	GdaDataSourceInfo *dsn_info;
-	GdaConnection *cnc;
-	static gint count = 0;
+	LoadedProvider *prv;
+	GdaConnection *cnc = NULL;
 	GList *l;
 
 	g_return_val_if_fail (GDA_IS_CLIENT (client), NULL);
@@ -480,33 +525,35 @@ gda_client_open_connection_from_string (GdaClient *client,
 		}
 	}
 
-	/* create a temporary DSNInfo */
-	dsn_info = g_new (GdaDataSourceInfo, 1);
-	dsn_info->name = g_strdup_printf ("GDA-Temporary-Data-Source-%d", count++);
-	dsn_info->provider = g_strdup (provider_id);
-	dsn_info->cnc_string = g_strdup (cnc_string);
-	dsn_info->description = g_strdup (_("Temporary data source created by libgda. Don't remove it"));
-	dsn_info->username = NULL;
-	dsn_info->password = NULL;
-
-	gda_config_save_data_source (dsn_info->name,
-				     dsn_info->provider,
-				     dsn_info->cnc_string,
-				     dsn_info->description,
-				     dsn_info->username,
-				     dsn_info->password, TRUE);
-
-	/* open the connection */
-	cnc = gda_client_open_connection (client,
-					  dsn_info->name,
-					  dsn_info->username,
-					  dsn_info->password,
-					  options,
-					  error);
-
-	/* free memory */
-	gda_config_remove_data_source (dsn_info->name);
-	gda_data_source_info_free (dsn_info);
+	/* try to find provider in our hash table */
+	if (provider_id) {
+		prv = g_hash_table_lookup (client->priv->providers, provider_id);
+		if (!prv)
+			prv = find_or_load_provider (client, provider_id);
+	
+		if (prv) {
+			cnc = g_object_new (GDA_TYPE_CONNECTION, 
+					    "client", client, 
+					    "provider_obj", prv->provider, 
+					    "cnc_string", cnc_string, 
+					    "username", username, 
+					    "password", password, 
+					    "options", options, NULL);
+			if (!gda_connection_open (cnc, error)) {
+				g_object_unref (cnc);
+				cnc = NULL;
+			}
+		}
+		else 
+			g_set_error (error, GDA_CLIENT_ERROR, 0, 
+				     _("Datasource configuration error: could not find provider '%s'"),
+				     provider_id);
+	}
+	else {
+		g_warning (_("Datasource configuration error: no provider specified"));
+		g_set_error (error, GDA_CLIENT_ERROR, 0, 
+			     _("Datasource configuration error: no provider specified"));
+	}
 
 	return cnc;
 }
@@ -519,7 +566,7 @@ gda_client_open_connection_from_string (GdaClient *client,
  * The GList returned is an internal pointer, so DON'T TRY TO
  * FREE IT.
  *
- * Returns: a GList of #GdaConnection objects.
+ * Returns: a GList of #GdaConnection objects; dont't modify that list
  */
 const GList *
 gda_client_get_connections (GdaClient *client)
@@ -595,17 +642,10 @@ void
 gda_client_close_all_connections (GdaClient *client)
 {
 	g_return_if_fail (GDA_IS_CLIENT (client));
+	g_return_if_fail (client->priv);
 
-	while (client->priv->connections != NULL) {
-		GdaConnection *cnc = client->priv->connections->data;
-
-		g_assert (GDA_IS_CONNECTION (cnc));
-
-		client->priv->connections = g_list_remove (client->priv->connections, cnc);
-		g_object_unref (cnc);
-	}
-
-	client->priv->connections = NULL;
+	if (client->priv->connections) 
+		g_list_foreach (client->priv->connections, (GFunc) gda_connection_close, NULL);
 }
 
 /**
@@ -645,15 +685,24 @@ void
 gda_client_notify_error_event (GdaClient *client, GdaConnection *cnc, GdaConnectionEvent *error)
 {
 	GdaParameterList *params;
+	GdaParameter *param;
+	GValue *value;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 	g_return_if_fail (error != NULL);
 
-	params = gda_parameter_list_new ();
-	gda_parameter_list_add_parameter (params, gda_parameter_new_gobject ("error", G_OBJECT (error)));
-	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_ERROR, params);
+	param = gda_parameter_new (G_TYPE_OBJECT);
+	gda_object_set_name (GDA_OBJECT (param), "error");
+	value = g_value_init (g_new0 (GValue, 1), G_TYPE_OBJECT);
+	g_value_set_object (value, G_OBJECT (error));
+	gda_parameter_set_value (param, value);
+	gda_value_free (value);
 
-	gda_parameter_list_free (params);
+	params = gda_parameter_list_new (NULL);
+	gda_parameter_list_add_param (params, param);
+	g_object_unref (param);
+	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_ERROR, params);
+	g_object_unref (params);
 }
 
 /**
@@ -703,16 +752,25 @@ void
 gda_client_notify_transaction_started_event (GdaClient *client, GdaConnection *cnc, GdaTransaction *xaction)
 {
 	GdaParameterList *params;
+	GdaParameter *param;
+	GValue *value;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 	g_return_if_fail (GDA_IS_TRANSACTION (xaction));
 
-	params = gda_parameter_list_new ();
-	gda_parameter_list_add_parameter (params, gda_parameter_new_gobject ("transaction", G_OBJECT (xaction)));
-	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_TRANSACTION_STARTED, params);
+	param = gda_parameter_new (G_TYPE_OBJECT);
+	gda_object_set_name (GDA_OBJECT (param), "transaction");
+	value = g_value_init (g_new0 (GValue, 1), G_TYPE_OBJECT);
+	g_value_set_object (value, G_OBJECT (xaction));
+	gda_parameter_set_value (param, value);
+	gda_value_free (value);
 
-	gda_parameter_list_free (params);
+	params = gda_parameter_list_new (NULL);
+	gda_parameter_list_add_param (params, param);
+	g_object_unref (param);
+	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_TRANSACTION_STARTED, params);
+	g_object_unref (params);
 }
 
 /**
@@ -728,16 +786,25 @@ void
 gda_client_notify_transaction_committed_event (GdaClient *client, GdaConnection *cnc, GdaTransaction *xaction)
 {
 	GdaParameterList *params;
+	GdaParameter *param;
+	GValue *value;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 	g_return_if_fail (GDA_IS_TRANSACTION (xaction));
 
-	params = gda_parameter_list_new ();
-	gda_parameter_list_add_parameter (params, gda_parameter_new_gobject ("transaction", G_OBJECT (xaction)));
-	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_TRANSACTION_COMMITTED, params);
+	param = gda_parameter_new (G_TYPE_OBJECT);
+	gda_object_set_name (GDA_OBJECT (param), "transaction");
+	value = g_value_init (g_new0 (GValue, 1), G_TYPE_OBJECT);
+	g_value_set_object (value, G_OBJECT (xaction));
+	gda_parameter_set_value (param, value);
+	gda_value_free (value);
 
-	gda_parameter_list_free (params);
+	params = gda_parameter_list_new (NULL);
+	gda_parameter_list_add_param (params, param);
+	g_object_unref (param);
+	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_TRANSACTION_COMMITTED, params);
+	g_object_unref (params);
 }
 
 /**
@@ -753,16 +820,25 @@ void
 gda_client_notify_transaction_cancelled_event (GdaClient *client, GdaConnection *cnc, GdaTransaction *xaction)
 {
 	GdaParameterList *params;
+	GdaParameter *param;
+	GValue *value;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 	g_return_if_fail (GDA_IS_TRANSACTION (xaction));
 
-	params = gda_parameter_list_new ();
-	gda_parameter_list_add_parameter (params, gda_parameter_new_gobject ("transaction", G_OBJECT (xaction)));
-	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_TRANSACTION_CANCELLED, params);
+	param = gda_parameter_new (G_TYPE_OBJECT);
+	gda_object_set_name (GDA_OBJECT (param), "transaction");
+	value = g_value_init (g_new0 (GValue, 1), G_TYPE_OBJECT);
+	g_value_set_object (value, G_OBJECT (xaction));
+	gda_parameter_set_value (param, value);
+	gda_value_free (value);
 
-	gda_parameter_list_free (params);
+	params = gda_parameter_list_new (NULL);
+	gda_parameter_list_add_param (params, param);
+	g_object_unref (param);
+	gda_client_notify_event (client, cnc, GDA_CLIENT_EVENT_TRANSACTION_CANCELLED, params);
+	g_object_unref (params);
 }
 
 /**
@@ -877,7 +953,7 @@ gda_client_rollback_transaction (GdaClient *client, GdaTransaction *xaction)
  * Get an XML string representing the parameters which can be present in the
  * DSN string used to open a connection.
  *
- * Returns: a string (free it after usage), or %NULL if an error occured
+ * Returns: a string (free it after usage), or %NULL if an error occurred
  *
  */
 gchar *
@@ -898,24 +974,19 @@ gda_client_get_dsn_specs (GdaClient *client, const gchar *provider)
 }
 
 /**
- * gda_client_get_specs_to_create_database
+ * gda_client_prepare_create_database
  * @client: a #GdaClient object.
  * @provider: a provider
- * @type: a #GdaClientSpecsType type
  *
- * Get an XML string representing the parameters required to perform a specific action
- * using the @provider provider. The action to perform is chosen by the @type argument.
- * 
- * For example to create a database, make a list of #GdaParameter
- * objects for each parameter listed in the returned XML string, and use it to call
- * gda_client_create_database().
+ * Creates a new #GdaServerOperation object which contains the specifications required
+ * to create a database. Once these specifications provided, use 
+ * gda_client_perform_create_database() to perform the database creation.
  *
- * Returns: a string (free it after usage), or %NULL if the provider does not implement database
+ * Returns: new #GdaServerOperation object, or %NULL if the provider does not support database
  * creation
  */
-gchar *
-gda_client_get_provider_specs (GdaClient *client, const gchar *provider,
-			       GdaClientSpecsType type)
+GdaServerOperation *
+gda_client_prepare_create_database (GdaClient *client, const gchar *provider)
 {
 	LoadedProvider *prv;
 
@@ -926,128 +997,102 @@ gda_client_get_provider_specs (GdaClient *client, const gchar *provider,
 
 	prv = find_or_load_provider (client, provider);
 	if (prv && prv->provider) 
-		return gda_server_provider_get_specs (prv->provider, type);
+		return gda_server_provider_create_operation (prv->provider, NULL, 
+							     GDA_SERVER_OPERATION_CREATE_DB, 
+							     NULL, NULL);
 	else
 		return NULL;
 }
 
 /**
- * gda_client_create_database
+ * gda_client_prepare_drop_database
  * @client: a #GdaClient object.
  * @provider: a provider
- * @params:
- * @error: a place to store en error, or %NULL
  *
- * Creates a new database using the specifications in @params. The list of parameters to
- * provide to create a database depends of the provider being used, and can be obtained using
- * the gda_client_get_specs_to_create_database() function.
+ * Creates a new #GdaServerOperation object which contains the specifications required
+ * to drop a database. Once these specifications provided, use 
+ * gda_client_perform_drop_database() to perform the database creation.
  *
- * For the providers which do not yet implement the gda_client_get_specs_to_create_database()
- * method, create a list with a #GdaParameter object for
- * <ul>
- *   <li>an opened #GdaConnection object, named "cnc"</li>
- *   <li>the name of the database to create, named "dbname"</li>
- * </ul>
- *
- * Returns: TRUE if no error occured and the database has been created
+ * Returns: new #GdaServerOperation object, or %NULL if the provider does not support database
+ * destruction
  */
-gboolean
-gda_client_create_database (GdaClient *client, const gchar *provider, GdaParameterList *params,
-			    GError **error)
+GdaServerOperation *
+gda_client_prepare_drop_database (GdaClient *client, const gchar *provider)
 {
 	LoadedProvider *prv;
 
-	g_return_val_if_fail (client && GDA_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (client && GDA_IS_CLIENT (client), NULL);
+
+	if (!provider || !*provider)
+		return NULL;
 
 	prv = find_or_load_provider (client, provider);
-	if (prv && prv->provider) {
-		/* see if we have only 2 parameters which are "cnc" and "name" */
-		if (params && (gda_parameter_list_get_length (params) == 2)) {
-			GdaParameter *cnc, *name;
-			
-			cnc = gda_parameter_list_find (params, "cnc");
-			name = gda_parameter_list_find (params, "dbname");
-			
-			if (name && cnc) {
-				const GdaValue *cnc_v, *name_v;
-				
-				cnc_v = gda_parameter_get_value (cnc);
-				name_v = gda_parameter_get_value (name);
-				return gda_server_provider_create_database_cnc (prv->provider, 
-										GDA_CONNECTION (gda_value_get_gobject (cnc_v)),
-										gda_value_get_string (name_v));
-			}
-		}
-		else
-			return gda_server_provider_perform_action_params (prv->provider, params, 
-									  GDA_CLIENT_SPECS_CREATE_DATABASE,
-									  error);
-	}
-	else {
-		g_set_error (error, GDA_CLIENT_ERROR, 0, 
-			     _("Could not find provider %s in the current setup"), provider); 
-		return FALSE;
-	}
-
-	return FALSE;
+	if (prv && prv->provider) 
+		return gda_server_provider_create_operation (prv->provider, NULL,
+							     GDA_SERVER_OPERATION_DROP_DB, 
+							     NULL, NULL);
+	else
+		return NULL;
 }
 
 /**
- * gda_client_drop_database
+ * gda_client_perform_create_database
  * @client: a #GdaClient object.
  * @provider: a provider
- * @dbname: the name of the database to destroy
+ * @op: a #GdaServerOperation object obtained using gda_client_prepare_server_operation()
  * @error: a place to store en error, or %NULL
  *
- * Destroys an existing database using the specifications in @params. The list of parameters to
- * provide to destroy a database depends of the provider being used, and can be obtained using
- * the gda_client_get_specs_to_drop_database() function.
+ * Creates a new database using the specifications in @op. 
  *
- * For the providers which do not yet implement the gda_client_get_specs_to_drop_database()
- * method, create a list with a #GdaParameter object for
- * <ul>
- *   <li>an opened #GdaConnection object, named "cnc"</li>
- *   <li>the name of the database to destroy, named "dbname"</li>
- * </ul>
- *
- * Returns: TRUE if no error occured and the database has been destroyed
+ * Returns: TRUE if no error occurred and the database has been created
  */
 gboolean
-gda_client_drop_database (GdaClient *client, const gchar *provider, 
-			  GdaParameterList *params, GError **error)
+gda_client_perform_create_database (GdaClient *client, const gchar *provider, GdaServerOperation *op,
+				    GError **error)
 {
 	LoadedProvider *prv;
 
-	g_return_val_if_fail (client && GDA_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (GDA_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
 
 	prv = find_or_load_provider (client, provider);
-	if (prv && prv->provider) {
-		/* see if we have only 2 parameters which are "cnc" and "name" */
-		if (params && (gda_parameter_list_get_length (params) == 2)) {
-			GdaParameter *cnc, *name;
-			
-			cnc = gda_parameter_list_find (params, "cnc");
-			name = gda_parameter_list_find (params, "dbname");
-			
-			if (name && cnc) {
-				const GdaValue *cnc_v, *name_v;
-				
-				cnc_v = gda_parameter_get_value (cnc);
-				name_v = gda_parameter_get_value (name);
-				return gda_server_provider_drop_database_cnc (prv->provider, 
-									      GDA_CONNECTION (gda_value_get_gobject (cnc_v)),
-									      gda_value_get_string (name_v));
-			}
-		}
-		else
-			return gda_server_provider_perform_action_params (prv->provider, params, 
-									  GDA_CLIENT_SPECS_CREATE_DATABASE, error);
+	if (prv && prv->provider) 
+		return gda_server_provider_perform_operation (prv->provider, NULL, 
+							      op, error);
+	else {
+		g_set_error (error, GDA_CLIENT_ERROR, 0, 
+			     _("Could not find provider %s in the current setup"), provider); 
+		return FALSE;
 	}
+}
+
+/**
+ * gda_client_perform_drop_database
+ * @client: a #GdaClient object.
+ * @provider: a provider
+ * @op: a #GdaServerOperation object obtained using gda_client_prepare_server_operation()
+ * @error: a place to store en error, or %NULL
+ *
+ * Destroys an existing database using the specifications in @op. 
+ *
+ * Returns: TRUE if no error occurred and the database has been destroyed
+ */
+gboolean
+gda_client_perform_drop_database (GdaClient *client, const gchar *provider, 
+				  GdaServerOperation *op, GError **error)
+{
+	LoadedProvider *prv;
+
+	g_return_val_if_fail (GDA_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
+
+	prv = find_or_load_provider (client, provider);
+	if (prv && prv->provider) 
+		return gda_server_provider_perform_operation (prv->provider, NULL, 
+							      op, error);
 	else {
 		g_set_error (error, GDA_CLIENT_ERROR, 0, 
 			     _("Could not find provider %s in the current setup"), provider); 
 		return FALSE;
 	}
-
-	return FALSE;
 }
