@@ -124,7 +124,7 @@ do_notify_changes (GdaDataModel *model)
 }
 
 /**
- * gda_data_model_changed
+ * gda_data_model_signal_emit_changed
  * @model: a #GdaDataModel object.
  *
  * Notifies listeners of the given data model object of changes
@@ -134,12 +134,12 @@ do_notify_changes (GdaDataModel *model)
  * or removed from the data model.
  */
 void
-gda_data_model_changed (GdaDataModel *model)
+gda_data_model_signal_emit_changed (GdaDataModel *model)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	
 	if (do_notify_changes (model)) 
-		gda_object_changed (GDA_OBJECT (model));
+		gda_object_signal_emit_changed (GDA_OBJECT (model));
 }
 
 /**
@@ -164,8 +164,8 @@ gda_data_model_row_inserted (GdaDataModel *model, gint row)
 		for (i = 0; i < nbcols; i++) {
 			column = gda_data_model_describe_column (model, i);
 			value = gda_data_model_get_value_at (model, i, 0);
-			if ((gda_column_get_gda_type (column) == G_TYPE_INVALID))
-				gda_column_set_gda_type (column, 
+			if ((gda_column_get_g_type (column) == G_TYPE_INVALID))
+				gda_column_set_g_type (column, 
 							 value ? G_VALUE_TYPE ((GValue *)value) : GDA_TYPE_NULL);
 		}
 	}
@@ -176,7 +176,7 @@ gda_data_model_row_inserted (GdaDataModel *model, gint row)
 			       gda_data_model_signals[ROW_INSERTED],
 			       0, row);
 
-		gda_data_model_changed (model);
+		gda_data_model_signal_emit_changed (model);
 	}
 }
 
@@ -197,7 +197,7 @@ gda_data_model_row_updated (GdaDataModel *model, gint row)
 			       gda_data_model_signals[ROW_UPDATED],
 			       0, row);
 
-		gda_data_model_changed (model);
+		gda_data_model_signal_emit_changed (model);
 	}
 }
 
@@ -218,7 +218,7 @@ gda_data_model_row_removed (GdaDataModel *model, gint row)
 			       gda_data_model_signals[ROW_REMOVED],
 			       0, row);
 
-		gda_data_model_changed (model);
+		gda_data_model_signal_emit_changed (model);
 	}
 }
 
@@ -362,6 +362,37 @@ gda_data_model_describe_column (GdaDataModel *model, gint col)
 	}
 }
 
+/**
+ * gda_data_model_get_column_index_by_name
+ * @model: a #GdaDataModel object.
+ * @name: a column name
+ *
+ * Get the index of the column named @name in @model
+ *
+ * Returns: the column index, or -1 if no column named @name was found
+ */
+gint
+gda_data_model_get_column_index_by_name (GdaDataModel *model, const gchar *name)
+{
+	gint nbcols, ncol, ret;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), -1);
+	g_return_val_if_fail (name, -1);
+	
+	ret = -1;
+	nbcols = gda_data_model_get_n_columns (model);
+	for(ncol = 0; ncol < nbcols; ncol++) {
+		if(g_str_equal (name, gda_data_model_get_column_title (model, ncol))) {
+			
+			ret = ncol;
+			break;
+		}
+	}
+	
+	return ret;
+}
+
+
 /**
  * gda_data_model_get_column_title
  * @model: a #GdaDataModel object.
@@ -436,6 +467,35 @@ gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row)
 	}
 }
 
+/**
+ * gda_data_model_get_value_at_col_name
+ * @model: a #GdaDataModel object.
+ * @col_name: a valid column name.
+ * @row: a valid row number.
+ *
+ * Retrieves the data stored in the given position (identified by
+ * the @col_name column and @row parameters) on a data model.
+ *
+ * See also gda_data_model_get_value_at().
+ *
+ * Returns: a #GValue containing the value stored in the given
+ * position, or %NULL on error (out-of-bound position, etc).
+ */
+const GValue *
+gda_data_model_get_value_at_col_name (GdaDataModel *model, 
+				      const gchar *column_name, gint row)
+{
+	gint ncol;
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
+	g_return_val_if_fail (column_name, NULL);
+
+	ncol = gda_data_model_get_column_index_by_name (model, column_name);
+	if (ncol == -1)
+		return NULL;
+	else
+		return gda_data_model_get_value_at (model, ncol, row);
+}
+
 /**
  * gda_data_model_get_attributes_at
  * @model: a #GdaDataModel object
@@ -447,7 +507,7 @@ gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row)
  * @row is -1, then the attributes returned correspond to a "would be" value
  * if a row was added to @model.
  *
- * Returns: the attributes
+ * Returns: the attributes as an ORed value of #GValueAttribute
  */
 guint
 gda_data_model_get_attributes_at (GdaDataModel *model, gint col, gint row)
@@ -901,7 +961,8 @@ gda_data_model_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GVal
 		(GDA_DATA_MODEL_GET_CLASS (model)->i_send_hint) (model, hint, hint_value);
 }
 
-static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, gchar sep);
+static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, 
+					const gint *rows, gint nb_rows, gchar sep);
 
 
 /**
@@ -910,6 +971,8 @@ static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, g
  * @format: the format in which to export data
  * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
+ * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
+ * @nb_rows: the number of rows in @rows
  * @options: list of options for the export
  *
  * Exports data contained in @model to a string; the format is specified using the @format argument.
@@ -925,7 +988,8 @@ static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, g
  */
 gchar *
 gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat format, 
-				 const gint *cols, gint nb_cols, GdaParameterList *options)
+				 const gint *cols, gint nb_cols, 
+				 const gint *rows, gint nb_rows, GdaParameterList *options)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 	g_return_val_if_fail (!options || GDA_IS_PARAMETER_LIST (options), NULL);
@@ -955,7 +1019,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 
 		g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 		
-		xml_node = gda_data_model_to_xml_node (model, cols, nb_cols, name);
+		xml_node = gda_data_model_to_xml_node (model, cols, nb_cols, rows, nb_rows, name);
 		xml_doc = xmlNewDoc ("1.0");
 		xmlDocSetRootElement (xml_doc, xml_node);
 		
@@ -990,7 +1054,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 		}
 
 		if (cols)
-			return export_to_text_separated (model, cols, nb_cols, sep);
+			return export_to_text_separated (model, cols, nb_cols, rows, nb_rows, sep);
 		else {
 			gchar *retval;
 			gint *rcols, rnb_cols, i;
@@ -1001,7 +1065,7 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
 			rcols = g_new (gint, rnb_cols);
 			for (i = 0; i < rnb_cols; i++)
 				rcols[i] = i;
-			retval = export_to_text_separated (model, rcols, rnb_cols, sep);
+			retval = export_to_text_separated (model, rcols, rnb_cols, rows, nb_rows, sep);
 			g_free (rcols);
 			
 			return retval;
@@ -1021,6 +1085,8 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
  * @file: the filename to export to
  * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
+ * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
+ * @nb_rows: the number of rows in @rows
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
  *
@@ -1041,6 +1107,7 @@ gboolean
 gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format, 
 			       const gchar *file,
 			       const gint *cols, gint nb_cols, 
+			       const gint *rows, gint nb_rows, 
 			       GdaParameterList *options, GError **error)
 {
 	gchar *body;
@@ -1050,7 +1117,7 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 	g_return_val_if_fail (!options || GDA_IS_PARAMETER_LIST (options), FALSE);
 	g_return_val_if_fail (file, FALSE);
 
-	body = gda_data_model_export_to_string (model, format, cols, nb_cols, options);
+	body = gda_data_model_export_to_string (model, format, cols, nb_cols, rows, nb_cols, options);
 	if (options) {
 		GdaParameter *param;
 		
@@ -1085,18 +1152,21 @@ gda_data_model_export_to_file (GdaDataModel *model, GdaDataModelIOFormat format,
 }
 
 static gchar *
-export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, gchar sep)
+export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, const gint *rows, gint nb_rows, gchar sep)
 {
 	GString *str;
 	gchar *retval;
-	gint c, rows, r;
+	gint c, nbrows, r;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
 	str = g_string_new ("");
-	rows = gda_data_model_get_n_rows (model);
+	if (!rows)
+		nbrows = gda_data_model_get_n_rows (model);
+	else
+		nbrows = nb_rows;
 
-	for (r = 0; r < rows; r++) {
+	for (r = 0; r < nbrows; r++) {
 		if (r > 0)
 			str = g_string_append_c (str, '\n');
 
@@ -1104,7 +1174,7 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols, g
 			GValue *value;
 			gchar *txt;
 
-			value = (GValue *) gda_data_model_get_value_at (model, cols[c], r);
+			value = (GValue *) gda_data_model_get_value_at (model, cols[c], rows ? rows[r] : r);
 			if (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)
 				txt = g_strdup (g_value_get_boolean (value) ? "TRUE" : "FALSE");
 			else
@@ -1145,6 +1215,8 @@ xml_set_int (xmlNodePtr node, const gchar *name, gint value)
  * @model: a #GdaDataModel object.
  * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
+ * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
+ * @nb_rows: the number of rows in @rows
  * @name: name to use for the XML resulting table.
  *
  * Converts a #GdaDataModel into a xmlNodePtr (as used in libxml).
@@ -1152,10 +1224,11 @@ xml_set_int (xmlNodePtr node, const gchar *name, gint value)
  * Returns: a xmlNodePtr representing the whole data model.
  */
 xmlNodePtr
-gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, const gchar *name)
+gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, 
+			    const gint *rows, gint nb_rows, const gchar *name)
 {
 	xmlNodePtr node;
-	gint rows, i;
+	gint i;
 	gint *rcols, rnb_cols;
 	gchar *arrayid;
 	gchar *str;
@@ -1189,7 +1262,6 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols,
 	}
 
 	/* set the table structure */
-	rows = gda_data_model_get_n_rows (model);
 	for (i = 0; i < rnb_cols; i++) {
 		GdaColumn *column;
 		xmlNodePtr field;
@@ -1216,7 +1288,7 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols,
 		cstr = gda_column_get_dbms_type (column);
 		if (cstr && *cstr)
 			xmlSetProp (field, "dbms_type", cstr);
-		xmlSetProp (field, "gdatype", gda_type_to_string (gda_column_get_gda_type (column)));
+		xmlSetProp (field, "gdatype", g_type_to_string (gda_column_get_g_type (column)));
 		if (gda_column_get_defined_size (column) != 0)
 			xml_set_int (field, "size", gda_column_get_defined_size (column));
 		if (gda_column_get_scale (column) != 0)
@@ -1238,7 +1310,7 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols,
 	}
 	
 	/* add the model data to the XML output */
-	utility_data_model_dump_data_to_xml (model, node, cols, nb_cols, FALSE);
+	utility_data_model_dump_data_to_xml (model, node, cols, nb_cols, rows, nb_rows, FALSE);
 
 	if (!cols)
 		g_free (rcols);
@@ -1333,7 +1405,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 				continue;
 		}
 
-		gdatype = gda_column_get_gda_type (column);
+		gdatype = gda_column_get_g_type (column);
 		if ((gdatype == G_TYPE_INVALID) ||
 		    (gdatype == GDA_TYPE_NULL)) {
 			g_set_error (error, 0, 0,
@@ -1355,7 +1427,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 			if (!gda_value_set_from_string (value, xmlNodeGetContent (xml_field), gdatype)) {
 				g_free (value);
 				g_set_error (error, 0, 0, _("Cannot interpret string as a valid %s value"), 
-					     gda_type_to_string (gdatype));
+					     g_type_to_string (gdatype));
 				retval = FALSE;
 				break;
 			}
@@ -1511,14 +1583,14 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 			return FALSE;
 		}
 		if (param) {
-			if (! g_value_type_transformable (gda_parameter_get_gda_type (param), 
-							  gda_column_get_gda_type (column))) {
+			if (! g_value_type_transformable (gda_parameter_get_g_type (param), 
+							  gda_column_get_g_type (column))) {
 				g_set_error (error, 0, 0,
 					     _("Destination column %d has a gda type (%s) incompatible with "
 					       "source column %d type (%s)"), i,
-					     gda_type_to_string (gda_column_get_gda_type (column)),
+					     g_type_to_string (gda_column_get_g_type (column)),
 					     col,
-					     gda_type_to_string (gda_parameter_get_gda_type (param)));
+					     g_type_to_string (gda_parameter_get_g_type (param)));
 				return FALSE;
 			}
 		}
@@ -1539,12 +1611,12 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 
 		column = gda_data_model_describe_column (to, i);
 		if (plist->data) {
-			if (gda_parameter_get_gda_type (GDA_PARAMETER (plist->data)) != 
-			    gda_column_get_gda_type (column)) {
+			if (gda_parameter_get_g_type (GDA_PARAMETER (plist->data)) != 
+			    gda_column_get_g_type (column)) {
 				GValue *newval;
 				
 				newval = g_new0 (GValue, 1);
-				append_types [i] = gda_column_get_gda_type (column);
+				append_types [i] = gda_column_get_g_type (column);
 				g_value_init (newval, append_types [i]);
 				append_values = g_list_prepend (append_values, newval);
 			}
@@ -1584,8 +1656,8 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 						g_set_error (error, 0, 0,
 							     _("Can't transform '%s' from GDA type %s to GDA type %s"),
 							     str, 
-							     gda_type_to_string (G_VALUE_TYPE (value)),
-							     gda_type_to_string (G_VALUE_TYPE ((GValue *) (avlist->data))));
+							     g_type_to_string (G_VALUE_TYPE (value)),
+							     g_type_to_string (G_VALUE_TYPE ((GValue *) (avlist->data))));
 						g_free (str);
 						retval = FALSE;
 					}
@@ -1757,7 +1829,7 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 #ifdef GDA_DEBUG_NO
 		{
 			GdaColumn *col = gda_data_model_describe_column (model, i);
-			g_print ("Model col %d has type %s\n", i, gda_type_to_string (gda_column_get_gda_type (col)));
+			g_print ("Model col %d has type %s\n", i, g_type_to_string (gda_column_get_g_type (col)));
 		}
 #endif
 
@@ -1768,8 +1840,8 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 			cols_size [i] = 6; /* for "<none>" */
 
 		gdacol = gda_data_model_describe_column (model, i);
-		coltype = gda_column_get_gda_type (gdacol);
-		/*g_string_append_printf (string, "COL %d is a %s\n", i+1, gda_type_to_string (coltype));*/
+		coltype = gda_column_get_g_type (gdacol);
+		/*g_string_append_printf (string, "COL %d is a %s\n", i+1, g_type_to_string (coltype));*/
 		if ((coltype == G_TYPE_INT64) ||
 		    (coltype == G_TYPE_UINT64) ||
 		    (coltype == G_TYPE_INT) ||
