@@ -1,6 +1,6 @@
 /* gda-dict-field.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -79,19 +79,21 @@ static GObjectClass  *parent_class = NULL;
 enum
 {
 	PROP_0,
-	PROP_DB_TABLE
+	PROP_DB_TABLE,
+	PROP_PLUGIN
 };
 
 
 /* private structure */
 struct _GdaDictFieldPrivate
 {
-	GdaDictType *data_type;
-	GdaDictTable          *table;
-	gint                   length;     /* -1 if not applicable */
-	gint                   scale;      /* 0 if not applicable */
-	GdaValue              *default_val;/* NULL if no default value */
-	guint                  extra_attrs;/* OR'ed value of GdaDictFieldAttribute */
+	GdaDictType     *data_type;
+	GdaDictTable    *table;
+	gint             length;     /* -1 if not applicable */
+	gint             scale;      /* 0 if not applicable */
+	GValue          *default_val;/* NULL if no default value */
+	guint            extra_attrs;/* OR'ed value of GdaDictFieldAttribute */
+	gchar           *plugin;
 };
 
 /* module error */
@@ -188,7 +190,9 @@ gda_dict_field_class_init (GdaDictFieldClass * class)
 	g_object_class_install_property (object_class, PROP_DB_TABLE,
 					 g_param_spec_pointer ("db_table", NULL, NULL, 
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-
+	g_object_class_install_property (object_class, PROP_PLUGIN,
+                                         g_param_spec_string ("entry_plugin", NULL, NULL, NULL,
+                                                              (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	/* virtual functions */
 #ifdef GDA_DEBUG
         GDA_OBJECT_CLASS (class)->dump = (void (*)(GdaObject *, guint)) gda_dict_field_dump;
@@ -205,6 +209,7 @@ gda_dict_field_init (GdaDictField * gda_dict_field)
 	gda_dict_field->priv->length = -1;
 	gda_dict_field->priv->scale = 0;
 	gda_dict_field->priv->extra_attrs = 0;
+	gda_dict_field->priv->plugin = NULL;
 }
 
 
@@ -231,7 +236,7 @@ gda_dict_field_new (GdaDict *dict, GdaDictType *type)
 	gda_dict_field = GDA_DICT_FIELD (obj);
 
 	if (type)
-		gda_dict_field_set_data_type (gda_dict_field, type);
+		gda_dict_field_set_dict_type (gda_dict_field, type);
 	
 	return obj;
 }
@@ -272,7 +277,7 @@ gda_dict_field_dispose (GObject *object)
 }
 
 static void
-gda_dict_field_finalize (GObject   * object)
+gda_dict_field_finalize (GObject *object)
 {
 	GdaDictField *gda_dict_field;
 
@@ -281,7 +286,8 @@ gda_dict_field_finalize (GObject   * object)
 
 	gda_dict_field = GDA_DICT_FIELD (object);
 	if (gda_dict_field->priv) {
-
+		g_free (gda_dict_field->priv->plugin);
+		
 		g_free (gda_dict_field->priv);
 		gda_dict_field->priv = NULL;
 	}
@@ -293,9 +299,9 @@ gda_dict_field_finalize (GObject   * object)
 
 static void 
 gda_dict_field_set_property (GObject *object,
-			guint param_id,
-			const GValue *value,
-			GParamSpec *pspec)
+			     guint param_id,
+			     const GValue *value,
+			     GParamSpec *pspec)
 {
 	gpointer ptr;
 	GdaDictField *gda_dict_field;
@@ -317,6 +323,13 @@ gda_dict_field_set_property (GObject *object,
 							 G_CALLBACK (destroyed_object_cb), gda_dict_field);
 			}
 			break;
+		case PROP_PLUGIN:
+			g_free (gda_dict_field->priv->plugin);
+			if (g_value_get_string (value))
+				gda_dict_field->priv->plugin = g_strdup (g_value_get_string (value));
+			else
+				gda_dict_field->priv->plugin = NULL;
+			break;
 		}
 	}
 }
@@ -335,6 +348,9 @@ gda_dict_field_get_property (GObject *object,
 		case PROP_DB_TABLE:
 			g_value_set_pointer (value, gda_dict_field->priv->table);
 			break;
+		case PROP_PLUGIN:
+			g_value_set_string (value, gda_dict_field->priv->plugin);
+			break;
 		}	
 	}
 }
@@ -366,7 +382,7 @@ gda_dict_field_set_length (GdaDictField *field, gint length)
 
 	/* signal the modification */
 	if (changed)
-		gda_object_changed (GDA_OBJECT (field));
+		gda_object_signal_emit_changed (GDA_OBJECT (field));
 }
 
 /**
@@ -411,7 +427,7 @@ gda_dict_field_set_scale (GdaDictField *field, gint scale)
 
 	/* signal the modification */
 	if (changed)
-		gda_object_changed (GDA_OBJECT (field));
+		gda_object_signal_emit_changed (GDA_OBJECT (field));
 }
 
 /**
@@ -463,14 +479,14 @@ gda_dict_field_get_constraints (GdaDictField *field)
 }
 
 /**
-* gda_dict_field_set_data_type
+* gda_dict_field_set_dict_type
 * @field: a #GdaDictField  object
 * @type: a #GdaDictType object
 *
 * Sets the data type of the field
 */
 void
-gda_dict_field_set_data_type (GdaDictField *field, GdaDictType *type)
+gda_dict_field_set_dict_type (GdaDictField *field, GdaDictType *type)
 {
 	g_return_if_fail (field && GDA_IS_DICT_FIELD (field));
 	g_return_if_fail (field->priv);
@@ -485,7 +501,7 @@ gda_dict_field_set_data_type (GdaDictField *field, GdaDictType *type)
 		gda_object_connect_destroy (type, G_CALLBACK (destroyed_object_cb), field);
 		
 		/* signal the modification */
-		gda_object_changed (GDA_OBJECT (field));
+		gda_object_signal_emit_changed (GDA_OBJECT (field));
 	}
 }
 
@@ -493,29 +509,29 @@ gda_dict_field_set_data_type (GdaDictField *field, GdaDictType *type)
 /**
  * gda_dict_field_set_default_value
  * @field: a #GdaDictField object
- * @value: a #GdaValue value or NULL
+ * @value: a #GValue value or NULL
  *
  * Sets (or replace) the default value for the field. WARNING: the default value's data type can be
  * different from the field's data type (this is the case for example if the default value is a 
  * function like Postgres's default value for the SERIAL data type).
  */
 void
-gda_dict_field_set_default_value (GdaDictField *field, const GdaValue *value)
+gda_dict_field_set_default_value (GdaDictField *field, const GValue *value)
 {
 	g_return_if_fail (field && GDA_IS_DICT_FIELD (field));
 	g_return_if_fail (field->priv);
 
-	if (gda_value_compare_ext (field->priv->default_val, value)) {
+	if (gda_value_compare_ext (field->priv->default_val, (GValue *) value)) {
 		if (field->priv->default_val) {
 			gda_value_free (field->priv->default_val);
 			field->priv->default_val = NULL;
 		}
 		
 		if (value)
-			field->priv->default_val = gda_value_copy (value);
+			field->priv->default_val = gda_value_copy ((GValue *) value);
 		
 		/* signal the modification */
-		gda_object_changed (GDA_OBJECT (field));
+		gda_object_signal_emit_changed (GDA_OBJECT (field));
 	}
 }
 
@@ -527,7 +543,7 @@ gda_dict_field_set_default_value (GdaDictField *field, const GdaValue *value)
  *
  * Returns: the default value
  */
-const GdaValue *
+const GValue *
 gda_dict_field_get_default_value (GdaDictField *field)
 {
 	g_return_val_if_fail (field && GDA_IS_DICT_FIELD (field), NULL);
@@ -839,10 +855,10 @@ gda_dict_field_save_to_xml (GdaXmlStorage *iface, GError **error)
 
 	if (field->priv->default_val) {
 		GdaDataHandler *dh;
-		GdaValueType vtype;
+		GType vtype;
 		
-		vtype = gda_value_get_type (field->priv->default_val);
-		xmlSetProp (node, "default_gda_type", gda_type_to_string (vtype));
+		vtype = G_VALUE_TYPE (field->priv->default_val);
+		xmlSetProp (node, "default_g_type", g_type_to_string (vtype));
 
 		dh = gda_dict_get_default_handler (gda_object_get_dict (GDA_OBJECT (field)), vtype);
 		str = gda_data_handler_get_str_from_value (dh, field->priv->default_val);
@@ -856,6 +872,9 @@ gda_dict_field_save_to_xml (GdaXmlStorage *iface, GError **error)
 		g_free (str);
 	}
 
+	if (field->priv->plugin && *(field->priv->plugin)) 
+		xmlSetProp (node, "plugin", field->priv->plugin);
+
 	return node;
 }
 
@@ -885,83 +904,116 @@ gda_dict_field_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **er
 	if (prop) {
 		name = TRUE;
 		gda_object_set_name (GDA_OBJECT (field), prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "descr");
 	if (prop) {
 		gda_object_set_description (GDA_OBJECT (field), prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "owner");
 	if (prop) {
 		gda_object_set_owner (GDA_OBJECT (field), prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "type");
 	if (prop) {
 		if ((*prop == 'D') && prop+1 && (*(prop+1) == 'T')) {
-			GdaDictType *dt = gda_dict_get_data_type_by_xml_id (dict, prop);
+			GdaDictType *dt = gda_dict_get_dict_type_by_xml_id (dict, prop);
 			if (dt) 
-				gda_dict_field_set_data_type (field, dt);
+				gda_dict_field_set_dict_type (field, dt);
 			else {
 				/* create a new custom data type */
-				gchar *tmp;
+				gchar *tmp, *tmp2;
+				xmlNodePtr tnode;
 				
 				dt = GDA_DICT_TYPE (gda_dict_type_new (dict));
 				tmp = utility_build_decoded_id (NULL, prop + 2);
 				gda_dict_type_set_sqlname (dt, tmp);
 				g_free (tmp);
-				gda_dict_type_set_gda_type (dt, GDA_VALUE_TYPE_BLOB);
-				gda_object_set_description (GDA_OBJECT (dt), _("Custom data type"));
-				gda_dict_declare_custom_data_type (dict, dt);
-				gda_dict_field_set_data_type (field, dt);
+				gda_dict_type_set_g_type (dt, GDA_TYPE_BLOB);
+
+				tnode = node->parent;
+				g_assert (tnode && !strcmp (tnode->name, "gda_dict_table"));
+				tmp = xmlGetProp (tnode, "name");
+				g_assert (tmp);
+				tmp2 = g_strdup_printf (_("Custom data type, declared for the %s.%s field"),
+							tmp, gda_object_get_name (GDA_OBJECT (field)));
+				xmlFree (tmp);
+				gda_object_set_description (GDA_OBJECT (dt), tmp2);
+				g_free (tmp2);
+				gda_dict_declare_object (dict, dt);
+				gda_dict_field_set_dict_type (field, dt);
 				g_object_unref (dt);
 			}
 			type = TRUE;
 		}
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "length");
 	if (prop) {
 		field->priv->length = atoi (prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "scale");
 	if (prop) {
 		field->priv->scale = atoi (prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 	
 	prop = xmlGetProp (node, "default");
 	if (prop) {
 		gchar *str2;
 
-		str2 = xmlGetProp (node, "default_gda_type");
+		str2 = xmlGetProp (node, "default_g_type");
 		if (str2) {
-			GdaValueType vtype;
+			GType vtype;
 			GdaDataHandler *dh;
-			GdaValue *value;
+			GValue *value;
 			
-			vtype = gda_type_from_string (str2);			
+			vtype = g_type_from_string (str2);
+			if (vtype == G_TYPE_INVALID) {
+				g_set_error (error,
+					     GDA_DICT_FIELD_ERROR,
+					     GDA_DICT_FIELD_XML_LOAD_ERROR,
+					     _("Unknown GDA data type '%s'"), str2);
+				xmlFree (str2);
+				return FALSE;				
+			}
 			dh = gda_dict_get_default_handler (dict, vtype);
-			value = gda_data_handler_get_value_from_str (dh, prop, vtype);
-			gda_dict_field_set_default_value (field, value);
-			gda_value_free (value);
-
-			g_free (str2);
+			if (dh) {
+				value = gda_data_handler_get_value_from_str (dh, prop, vtype);
+				gda_dict_field_set_default_value (field, value);
+				gda_value_free (value);
+			}
+			else {
+				g_set_error (error,
+					     GDA_DICT_FIELD_ERROR,
+					     GDA_DICT_FIELD_XML_LOAD_ERROR,
+					     _("Could not find a data handler for data type '%s'"), str2);
+				xmlFree (str2);
+				return FALSE;
+			}
+			xmlFree (str2);
 		}
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "extra_attr");
 	if (prop) {
 		gda_dict_field_set_attributes (field, utility_table_field_attrs_parse (prop));
-		g_free (prop);
+		xmlFree (prop);
+	}
+
+	prop = xmlGetProp (node, "plugin");
+	if (prop) {
+		g_object_set (G_OBJECT (field), "entry_plugin", prop, NULL);
+		xmlFree (prop);
 	}
 
 	if (name && type)
