@@ -1,6 +1,6 @@
 /* gda-graphviz.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -372,7 +372,7 @@ get_depend_query (GdaQuery *query, GSList **deplist, GSList **alldisplayed)
 	list = gda_query_get_sub_queries (query);
 	*alldisplayed = g_slist_concat (*alldisplayed, list);
 
-	list = g_slist_copy (gda_query_get_param_sources (query));
+	list = g_slist_copy ((GSList *) gda_query_get_param_sources (query));
 	*alldisplayed = g_slist_concat (*alldisplayed, list);
 
 	list = gda_query_get_all_fields (query);
@@ -545,7 +545,7 @@ do_graph_query (GdaGraphviz *graph, GString *string, GdaQuery *query, gint tabof
 			fid = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (list->data));
 			g_string_append_printf (string, "%s\t\"%s\" ", str, fid);
 			if (gda_referer_is_active (GDA_REFERER (list->data)))
-				g_string_append_printf (string, "[shape=record, label=\"%s\", style=filled, fillcolor=light_blue];\n",
+				g_string_append_printf (string, "[shape=record, label=\"%s\", style=filled, fillcolor=deepskyblue1];\n",
 							label);
 			else
 				g_string_append_printf (string, "[shape=record, label=\"%s\", style=filled, fillcolor=indianred];\n",
@@ -735,8 +735,10 @@ render_qf_field_label (GdaGraphviz *graph, GdaQueryFieldField *field)
 	if (cstr)
 		g_string_append (retval, cstr);
 	ref = gda_query_field_field_get_ref_field (field);
-	if (ref)
-		g_string_append_printf (retval, " (%s)", gda_object_get_name (GDA_OBJECT (ref)));
+	if (ref) {
+		if (!cstr || strcmp (cstr, gda_object_get_name (GDA_OBJECT (ref))))
+			g_string_append_printf (retval, " (%s)", gda_object_get_name (GDA_OBJECT (ref)));
+	}
 
 	g_string_append (retval, " | {Field");
 
@@ -761,7 +763,7 @@ render_qf_value_label (GdaGraphviz *graph, GdaQueryFieldValue *field)
 	GString *retval;
 	gchar *str;
 	const gchar *cstr;
-	const GdaValue *value;
+	const GValue *value;
 
 	retval = g_string_new ("{");
 	cstr = gda_object_get_name (GDA_OBJECT (field));
@@ -769,7 +771,7 @@ render_qf_value_label (GdaGraphviz *graph, GdaQueryFieldValue *field)
 		g_string_append (retval, cstr);
 	value = gda_query_field_value_get_value (field);
 	if (value) {
-		str = gda_value_stringify (value);
+		str = gda_value_stringify ((GValue *) value);
 		g_string_append_printf (retval, " (%s)", str);
 		g_free (str);
 	}
