@@ -1,6 +1,6 @@
 /* gda-query-field-func.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -260,7 +260,7 @@ gda_query_field_func_init (GdaQueryFieldFunc *gda_query_field_func)
  *
  * Returns: the new object
  */
-GObject*
+GdaQueryField*
 gda_query_field_func_new (GdaQuery *query, const gchar *func_name)
 {
 	GObject *obj;
@@ -271,7 +271,7 @@ gda_query_field_func_new (GdaQuery *query, const gchar *func_name)
 	obj = g_object_new (GDA_TYPE_QUERY_FIELD_FUNC, "dict", gda_object_get_dict (GDA_OBJECT (query)), 
 			    "query", query, "function_name", func_name, NULL);
 
-	return obj;
+	return (GdaQueryField *) obj;
 }
 
 static void 
@@ -441,6 +441,10 @@ gda_query_field_func_copy (GdaQueryField *orig)
 		GdaObjectRefType ref_type;
 		GType ref_gtype;
 
+		ref_str = gda_object_ref_get_ref_object_name (qf->priv->func_ref);
+		if (ref_str)
+			g_object_set (G_OBJECT (GDA_QUERY_FIELD_FUNC (obj)->priv->func_ref), "obj_name", ref_str, NULL);
+
 		ref_str = gda_object_ref_get_ref_name (qf->priv->func_ref, &ref_gtype, &ref_type);
 		if (ref_str)
 			gda_object_ref_set_ref_name (GDA_QUERY_FIELD_FUNC (obj)->priv->func_ref, ref_gtype, ref_type, ref_str);
@@ -559,7 +563,7 @@ gda_query_field_func_set_args (GdaQueryFieldFunc *func, GSList *args)
 	if (args && gda_object_ref_activate (func->priv->func_ref)) {
 		/* test for the right arguments */
 		GdaDictFunction *sfunc = GDA_DICT_FUNCTION (gda_object_ref_get_ref_object (func->priv->func_ref));
-		GSList *arg = args, *list = (GSList *) gda_dict_function_get_arg_types (sfunc);
+		GSList *arg = args, *list = (GSList *) gda_dict_function_get_arg_dict_types (sfunc);
 		GdaServerProviderInfo *sinfo = NULL;
 		GdaDict *dict;
 		GdaConnection *cnc;
@@ -581,11 +585,11 @@ gda_query_field_func_set_args (GdaQueryFieldFunc *func, GSList *args)
 					args_ok = FALSE;
 			}
 			else {
-				/* GdaValueType compatibility test */
+				/* GType compatibility test */
 				if (arg->data && list->data &&
 				    list->data &&
-				    (gda_dict_type_get_gda_type (gda_query_field_get_dict_type (GDA_QUERY_FIELD (arg->data))) !=
-				     gda_dict_type_get_gda_type (GDA_DICT_TYPE (list->data))))
+				    (gda_dict_type_get_g_type (gda_query_field_get_dict_type (GDA_QUERY_FIELD (arg->data))) !=
+				     gda_dict_type_get_g_type (GDA_DICT_TYPE (list->data))))
 					args_ok = FALSE;
 			}
 
@@ -713,7 +717,7 @@ gda_query_field_func_get_data_type (GdaEntityField *iface)
 	if (gda_query_field_func_activate (GDA_REFERER (iface))) {
 		GdaDictFunction *func;
 		func = GDA_DICT_FUNCTION (gda_object_ref_get_ref_object (GDA_QUERY_FIELD_FUNC (iface)->priv->func_ref));
-		return gda_dict_function_get_ret_type (func);
+		return gda_dict_function_get_ret_dict_type (func);
 	}
 
 	return NULL;
@@ -894,7 +898,7 @@ gda_query_field_func_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GErro
 		ref = gda_object_ref_get_ref_object (func->priv->func_ref);
 		if (ref && 
 		    (g_slist_length (func->priv->args) != 
-		     g_slist_length ((GSList *) gda_dict_function_get_arg_types (GDA_DICT_FUNCTION (ref))))) {
+		     g_slist_length ((GSList *) gda_dict_function_get_arg_dict_types (GDA_DICT_FUNCTION (ref))))) {
 			g_set_error (error,
 				     GDA_QUERY_FIELD_FUNC_ERROR,
 				     GDA_QUERY_FIELD_FUNC_XML_LOAD_ERROR,
